'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _asyncGenerator2 = require('babel-runtime/helpers/asyncGenerator');

var _asyncGenerator3 = _interopRequireDefault(_asyncGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = require('debug')('git3po:github');
const Promise = require('bluebird');
const request = require('superagent-bluebird-promise');

class GitHubApi {
  constructor(url, token, repo, dryRun) {
    this._url = url;
    this._token = token;
    this._repo = repo;
    this._dryRun = dryRun;
  }

  get url() {
    return `${this._url}/repos/${this._repo}`;
  }

  _authenticatedRequest(url, method = 'get') {
    return request[method](url).set('Authorization', `token ${this._token}`);
  }

  _fetchUntilFinished(request, onEach) {
    var _this = this;

    return _asyncGenerator3.default.wrap(function* () {
      let merged = [];
      let pages = 0;
      while (request) {
        pages++;

        log('fetching page %d - %d results so far', pages, merged.length);
        const response = yield _asyncGenerator3.default.await(request); // eslint-disable-line no-await-in-loop
        const body = response.body;
        const next = response.links && response.links.next;

        if (onEach) {
          yield _asyncGenerator3.default.await(onEach(body)); // eslint-disable-line no-await-in-loop
        }

        yield _asyncGenerator3.default.await((yield body)); // eslint-disable-line no-await-in-loop
        merged = merged.concat(body);
        request = next ? _this._authenticatedRequest(next).promise() : null;
      }

      log('fetched %d pages - %d total results', pages, merged.length);
      return merged;
    })();
  }

  fetchIssuesSince(date, state = 'all') {
    const url = `${this.url}/issues`;
    const since = date.toISOString();
    log(`fetching issues since=${since} state=${state}`);

    const query = this._authenticatedRequest(url).query({ since, state }).promise();
    return this._fetchUntilFinished(query);
  }

  fetchPullRequestsSince(date, state = 'all') {
    var _this2 = this;

    const url = `${this.url}/pulls`;
    const since = date.toISOString();
    log(`fetching PRs since=${since} state=${state}`);

    const query = this._authenticatedRequest(url).query({ since, state }).promise();
    return this._fetchUntilFinished(query, page => {
      return Promise.map(page, (() => {
        var _ref = (0, _asyncToGenerator3.default)(function* (prItem) {
          const index = page.indexOf(prItem);
          const pr = yield _this2.fetchPullRequest(prItem.number);
          const issue = yield _this2.fetchIssue(prItem.number);
          page[index] = (0, _assign2.default)({}, issue, pr);
        });

        return function (_x) {
          return _ref.apply(this, arguments);
        };
      })());
    });
  }

  fetchPullRequest(number) {
    var _this3 = this;

    return (0, _asyncToGenerator3.default)(function* () {
      const url = `${_this3.url}/pulls/${number}`;
      log(`fetching pr #${number}`);
      const response = yield _this3._authenticatedRequest(url).promise();
      return response.body;
    })();
  }

  fetchIssue(number) {
    var _this4 = this;

    return (0, _asyncToGenerator3.default)(function* () {
      const url = `${_this4.url}/issues/${number}`;
      log(`fetching issue #${number}`);
      const response = yield _this4._authenticatedRequest(url).promise();
      return response.body;
    })();
  }

  fetchComments(issue) {
    var _this5 = this;

    return (0, _asyncToGenerator3.default)(function* () {
      const url = `${_this5.url}/issues/${issue.number}/comments`;
      log(`fetching comments for issue=${issue.number}`);
      const response = yield _this5._authenticatedRequest(url).promise();

      if (issue.pull_request) {
        log(`fetching comments for PR=${issue.number}`);
        const reviewCommentsUrl = url.replace('/issues/', '/pulls/');
        const reviewResponse = yield _this5._authenticatedRequest(reviewCommentsUrl).promise();
        return response.body.concat(reviewResponse.body);
      } else {
        return response.body;
      }
    })();
  }

  fetchCommits(pullRequest) {
    var _this6 = this;

    return (0, _asyncToGenerator3.default)(function* () {
      const url = `${_this6.url}/pulls/${pullRequest.number}/commits`;
      log(`fetching commits for PR=${pullRequest.number}`);
      const response = yield _this6._authenticatedRequest(url).promise();
      return response.body;
    })();
  }

  dryRunGuardedRequest(url, method, action) {
    var _this7 = this;

    return (0, _asyncToGenerator3.default)(function* () {
      if (_this7._dryRun) {
        log(`dry run, skipping ${method.toUpperCase()} request to ${url}`);
        return Promise.resolve();
      }

      let promiseLike = action(_this7._authenticatedRequest(`${_this7.url}${url}`, method));
      if (typeof promiseLike.promise === 'function') {
        promiseLike = promiseLike.promise();
      }

      const result = yield promiseLike;
      log(`received successful response from ${url}`);
      return result;
    })();
  }

  addComment(issue, comment) {
    return this.dryRunGuardedRequest(`/issues/${issue.number}/comments`, 'post', request => request.type('json').send({ body: comment }));
  }

  addLabel(issue, label) {
    const labels = issue.labels.map(l => l.name).concat(label);
    return this.dryRunGuardedRequest(`/issues/${issue.number}`, 'patch', request => request.type('json').send({ labels })).then(response => {
      issue.labels.push({ name: label });
      return response;
    });
  }

  removeLabel(issue, label) {
    const labels = issue.labels.map(l => l.name).filter(name => name !== label);
    return this.dryRunGuardedRequest(`/issues/${issue.number}`, 'patch', request => request.type('json').send({ labels })).then(response => {
      issue.labels = issue.labels.filter(l => l.name !== label);
      return response;
    });
  }

  closeIssue(issue) {
    return this.dryRunGuardedRequest(`/issues/${issue.number}`, 'patch', request => request.type('json').send({ state: 'closed' }));
  }

  lockIssue(issue, lockReason) {
    return this.dryRunGuardedRequest(`/issues/${issue.number}/lock`, 'put',
    // eslint-disable-next-line camelcase
    request => request.type('json').send({ lock_reason: lockReason || 'resolved' }));
  }

  mergePullRequest(pr, mergeMethod, title, message) {
    return this.dryRunGuardedRequest(`/pulls/${pr.number}/merge`, 'put',
    /* eslint-disable camelcase */
    request => request.type('json').send({
      sha: pr.head.sha,
      merge_method: mergeMethod || 'rebase',
      commit_title: title,
      commit_message: message
    })
    /* eslint-enable camelcase */
    );
  }

  static transformIssue(issue) {
    return {
      text: [issue.title, issue.body].join('\n'),
      title: issue.title,
      body: issue.body,
      state: issue.state,
      locked: issue.locked,
      author: issue.user && issue.user.login,
      mergeable: issue.mergeable,
      mergeableState: issue.mergeable_state,
      labels: (issue.labels || []).map(label => label.name),
      updatedAt: new Date(issue.updated_at)
    };
  }

  static transformCommentOrCommit(item) {
    return item.commit ? GitHubApi.transformCommit(item) : GitHubApi.transformComment(item);
  }

  static transformComment(comment) {
    return {
      type: 'comment',
      text: comment.body,
      author: comment.user.login,
      createdAt: new Date(comment.created_at)
    };
  }

  static transformCommit(commit) {
    return {
      type: 'commit',
      text: commit.commit.message,
      author: commit.author.login,
      createdAt: new Date(commit.commit.committer.date)
    };
  }
}

module.exports = GitHubApi;